@@ -1,27 +1,38 @@
 """Attributes common to PolyData and Grid Objects."""
 
-import warnings
 import collections.abc
 import logging
-from typing import Optional, List, Tuple, Iterable, Union, Any, Dict
-from typing_extensions import Literal # for python versions <3.8
+import sys
+from typing import Any, Dict, Iterable, List, Optional, Sequence, Tuple, Union
+import warnings
+
+if sys.version_info >= (3, 8):
+    from typing import Literal
+else:
+    from typing_extensions import Literal
 
 import numpy as np
 
 import pyvista
 from pyvista import _vtk
-from pyvista.utilities import (FieldAssociation, get_array,
-                               get_array_association,
-                               is_pyvista_dataset, raise_not_matching,
-                               vtk_id_list_to_array, abstract_class,
-                               axis_rotation, transformations)
-from pyvista.utilities.misc import PyvistaDeprecationWarning
+from pyvista.utilities import (
+    FieldAssociation,
+    abstract_class,
+    get_array,
+    get_array_association,
+    is_pyvista_dataset,
+    raise_not_matching,
+    transformations,
+    vtk_id_list_to_array,
+)
 from pyvista.utilities.errors import check_valid_vector
+from pyvista.utilities.misc import PyvistaDeprecationWarning
+
+from .._typing import Vector
 from .dataobject import DataObject
 from .datasetattributes import DataSetAttributes
 from .filters import DataSetFilters, _get_output
 from .pyvista_ndarray import pyvista_ndarray
-from .._typing import Vector
 
 log = logging.getLogger(__name__)
 log.setLevel('CRITICAL')
@@ -107,7 +118,7 @@ class DataSet(DataSetFilters, DataObject):
 
         Association refers to the data association (e.g. point, cell, or
         field) of the active scalars.
-        
+
         Returns
         -------
         ActiveArrayInfo
@@ -163,7 +174,7 @@ class DataSet(DataSetFilters, DataObject):
 
         Association refers to the data association (e.g. point, cell, or
         field) of the active vectors.
-        
+
         Returns
         -------
         ActiveArrayInfo
@@ -329,7 +340,7 @@ class DataSet(DataSetFilters, DataObject):
         array.
 
         >>> import pyvista
-        >>> cube = pyvista.Cube().clean()
+        >>> cube = pyvista.Cube()
         >>> points = cube.points
         >>> points
         pyvista_ndarray([[-0.5, -0.5, -0.5],
@@ -420,7 +431,7 @@ class DataSet(DataSetFilters, DataObject):
         plot the active vectors.
 
         >>> import pyvista
-        >>> mesh = pyvista.Cube().clean()
+        >>> mesh = pyvista.Cube()
         >>> mesh_w_normals = mesh.compute_normals()
         >>> mesh_w_normals.active_vectors_name = 'Normals'
         >>> arrows = mesh_w_normals.arrows
@@ -538,7 +549,7 @@ class DataSet(DataSetFilters, DataObject):
         >>> from pyvista import examples
         >>> globe = examples.load_globe()
         >>> globe.textures
-        {'2k_earth_daymap': (Texture)...}
+        {'2k_earth_daymap': ...}
 
         """
         return self._textures
@@ -553,7 +564,7 @@ class DataSet(DataSetFilters, DataObject):
         >>> from pyvista import examples
         >>> globe = examples.load_globe()
         >>> globe.textures
-        {'2k_earth_daymap': (Texture)...}
+        {'2k_earth_daymap': ...}
         >>> globe.clear_textures()
         >>> globe.textures
         {}
@@ -739,7 +750,7 @@ class DataSet(DataSetFilters, DataObject):
 
         >>> import pyvista
         >>> import numpy as np
-        >>> cube = pyvista.Cube().clean()
+        >>> cube = pyvista.Cube()
         >>> cube['my_array'] = range(cube.n_points)
         >>> cube.rename_array('my_array', 'my_renamed_array')
         >>> cube['my_renamed_array']
@@ -850,29 +861,20 @@ class DataSet(DataSetFilters, DataObject):
         # Use the array range
         return np.nanmin(arr), np.nanmax(arr)
 
-    def points_to_double(self):
-        """Convert the points datatype to double precision.
-
-        Examples
-        --------
-        Create a mesh that has points of the type ``float32`` and
-        convert the points to ``float64``.
-
-        >>> import pyvista
-        >>> mesh = pyvista.Sphere()
-        >>> mesh.points.dtype
-        dtype('float32')
-        >>> mesh.points_to_double()
-        >>> mesh.points.dtype
-        dtype('float64')
-
-        """
-        if self.points.dtype != np.double:
-            self.points = self.points.astype(np.double)
-
-    def rotate_x(self, angle: float, point=None, transform_all_input_vectors=False):
+    def rotate_x(
+            self,
+            angle: float,
+            point=(0.0, 0.0, 0.0),
+            transform_all_input_vectors=False,
+            inplace=False
+    ):
         """Rotate mesh about the x-axis.
 
+        .. note::
+            See also the notes at :func:`transform()
+            <DataSetFilters.transform>` which is used by this filter
+            under the hood.
+
         Parameters
         ----------
         angle : float
@@ -885,17 +887,50 @@ class DataSet(DataSetFilters, DataObject):
             When ``True``, all input vectors are
             transformed. Otherwise, only the points, normals and
             active vectors are transformed.
-        """
-        if point is None:
-            point = (0.0, 0.0, 0.0)
-        if len(point) != 3:
-            raise ValueError('Point must be a vector of 3 values.')
-        t = transformations.axis_angle_rotation((1, 0, 0), angle, point=point, deg=True)
-        self.transform(t, transform_all_input_vectors=transform_all_input_vectors, inplace=True)
 
-    def rotate_y(self, angle: float, point=None, transform_all_input_vectors=False):
+        inplace : bool, optional
+            Updates mesh in-place.
+
+        Returns
+        -------
+        pyvista.DataSet
+            Rotated dataset.
+
+        Examples
+        --------
+        Rotate a mesh 30 degrees about the x-axis.
+
+        >>> import pyvista
+        >>> mesh = pyvista.Cube()
+        >>> rot = mesh.rotate_x(30, inplace=False)
+
+        Plot the rotated mesh.
+
+        >>> pl = pyvista.Plotter()
+        >>> _ = pl.add_mesh(rot)
+        >>> _ = pl.add_mesh(mesh, style='wireframe', line_width=3)
+        >>> _ = pl.add_axes_at_origin()
+        >>> pl.show()
+
+        """
+        check_valid_vector(point, "point")
+        t = transformations.axis_angle_rotation((1, 0, 0), angle, point=point, deg=True)
+        return self.transform(t, transform_all_input_vectors=transform_all_input_vectors, inplace=inplace)
+
+    def rotate_y(
+            self,
+            angle: float,
+            point=(0.0, 0.0, 0.0),
+            transform_all_input_vectors=False,
+            inplace=False
+    ):
         """Rotate mesh about the y-axis.
 
+        .. note::
+            See also the notes at :func:`transform()
+            <DataSetFilters.transform>` which is used by this filter
+            under the hood.
+
         Parameters
         ----------
         angle : float
@@ -908,17 +943,50 @@ class DataSet(DataSetFilters, DataObject):
             When ``True``, all input vectors are
             transformed. Otherwise, only the points, normals and
             active vectors are transformed.
-        """
-        if point is None:
-            point = (0.0, 0.0, 0.0)
-        if len(point) != 3:
-            raise ValueError('Point must be a vector of 3 values.')
-        t = transformations.axis_angle_rotation((0, 1, 0), angle, point=point, deg=True)
-        self.transform(t, transform_all_input_vectors=transform_all_input_vectors, inplace=True)
 
-    def rotate_z(self, angle: float, point=None, transform_all_input_vectors=False):
+        inplace : bool, optional
+            Updates mesh in-place.
+
+        Returns
+        -------
+        pyvista.DataSet
+            Rotated dataset.
+
+        Examples
+        --------
+        Rotate a cube 30 degrees about the y-axis.
+
+        >>> import pyvista
+        >>> mesh = pyvista.Cube()
+        >>> rot = mesh.rotate_y(30, inplace=False)
+
+        Plot the rotated mesh.
+
+        >>> pl = pyvista.Plotter()
+        >>> _ = pl.add_mesh(rot)
+        >>> _ = pl.add_mesh(mesh, style='wireframe', line_width=3)
+        >>> _ = pl.add_axes_at_origin()
+        >>> pl.show()
+
+        """
+        check_valid_vector(point, "point")
+        t = transformations.axis_angle_rotation((0, 1, 0), angle, point=point, deg=True)
+        return self.transform(t, transform_all_input_vectors=transform_all_input_vectors, inplace=inplace)
+
+    def rotate_z(
+            self,
+            angle: float,
+            point=(0.0, 0.0, 0.0),
+            transform_all_input_vectors=False,
+            inplace=False
+    ):
         """Rotate mesh about the z-axis.
 
+        .. note::
+            See also the notes at :func:`transform()
+            <DataSetFilters.transform>` which is used by this filter
+            under the hood.
+
         Parameters
         ----------
         angle : float
@@ -931,21 +999,54 @@ class DataSet(DataSetFilters, DataObject):
             When ``True``, all input vectors are
             transformed. Otherwise, only the points, normals and
             active vectors are transformed.
-        """
-        if point is None:
-            point = (0.0, 0.0, 0.0)
-        if len(point) != 3:
-            raise ValueError('Point must be a vector of 3 values.')
-        t = transformations.axis_angle_rotation((0, 0, 1), angle, point=point, deg=True)
-        self.transform(t, transform_all_input_vectors=transform_all_input_vectors, inplace=True)
 
-    def rotate_vector(self, vector: List[float], angle, point=None,
-                      transform_all_input_vectors=False):
+        inplace : bool, optional
+            Updates mesh in-place.
+
+        Returns
+        -------
+        pyvista.DataSet
+            Rotated dataset.
+
+        Examples
+        --------
+        Rotate a mesh 30 degrees about the z-axis.
+
+        >>> import pyvista
+        >>> mesh = pyvista.Cube()
+        >>> rot = mesh.rotate_z(30, inplace=False)
+
+        Plot the rotated mesh.
+
+        >>> pl = pyvista.Plotter()
+        >>> _ = pl.add_mesh(rot)
+        >>> _ = pl.add_mesh(mesh, style='wireframe', line_width=3)
+        >>> _ = pl.add_axes_at_origin()
+        >>> pl.show()
+
+        """
+        check_valid_vector(point, "point")
+        t = transformations.axis_angle_rotation((0, 0, 1), angle, point=point, deg=True)
+        return self.transform(t, transform_all_input_vectors=transform_all_input_vectors, inplace=inplace)
+
+    def rotate_vector(
+            self,
+            vector: Iterable[float],
+            angle: float,
+            point=(0.0, 0.0, 0.0),
+            transform_all_input_vectors=False,
+            inplace=False
+    ):
         """Rotate mesh about a vector.
 
+        .. note::
+            See also the notes at :func:`transform()
+            <DataSetFilters.transform>` which is used by this filter
+            under the hood.
+
         Parameters
         ----------
-        vector : tuple
+        vector : Iterable
             Axes to rotate about.
 
         angle : float
@@ -958,24 +1059,63 @@ class DataSet(DataSetFilters, DataObject):
             When ``True``, all input vectors are
             transformed. Otherwise, only the points, normals and
             active vectors are transformed.
-        """
-        if point is None:
-            point = (0.0, 0.0, 0.0)
-        if len(vector) != 3:
-            raise ValueError('Vector must be a vector of 3 values.')
-        if len(point) != 3:
-            raise ValueError('Point must be a vector of 3 values.')
-        t = transformations.axis_angle_rotation(vector, angle, point=point, deg=True)
-        self.transform(t, transform_all_input_vectors=transform_all_input_vectors, inplace=True)
 
-    def translate(self, xyz: Union[list, tuple, np.ndarray]):
+        inplace : bool, optional
+            Updates mesh in-place.
+
+        Returns
+        -------
+        pyvista.DataSet
+            Rotated dataset.
+
+        Examples
+        --------
+        Rotate a mesh 30 degrees about the ``(1, 1, 1)`` axis.
+
+        >>> import pyvista
+        >>> mesh = pyvista.Cube()
+        >>> rot = mesh.rotate_vector((1, 1, 1), 30, inplace=False)
+
+        Plot the rotated mesh.
+
+        >>> pl = pyvista.Plotter()
+        >>> _ = pl.add_mesh(rot)
+        >>> _ = pl.add_mesh(mesh, style='wireframe', line_width=3)
+        >>> _ = pl.add_axes_at_origin()
+        >>> pl.show()
+
+        """
+        check_valid_vector(vector)
+        check_valid_vector(point, "point")
+        t = transformations.axis_angle_rotation(vector, angle, point=point, deg=True)
+        return self.transform(t, transform_all_input_vectors=transform_all_input_vectors, inplace=inplace)
+
+    def translate(self, xyz: Union[list, tuple, np.ndarray], transform_all_input_vectors=False, inplace=False):
         """Translate the mesh.
 
+        .. note::
+            See also the notes at :func:`transform()
+            <DataSetFilters.transform>` which is used by this filter
+            under the hood.
+
         Parameters
         ----------
         xyz : list or tuple or np.ndarray
             Length 3 list, tuple or array.
 
+        transform_all_input_vectors : bool, optional
+            When ``True``, all input vectors are
+            transformed. Otherwise, only the points, normals and
+            active vectors are transformed.
+
+        inplace : bool, optional
+            Updates mesh in-place.
+
+        Returns
+        -------
+        pyvista.DataSet
+            Translated dataset.
+
         Examples
         --------
         Create a sphere and translate it by ``(2, 1, 2)``.
@@ -984,21 +1124,41 @@ class DataSet(DataSetFilters, DataObject):
         >>> mesh = pyvista.Sphere()
         >>> mesh.center
         [0.0, 0.0, 0.0]
-        >>> mesh.translate((2, 1, 2))
-        >>> mesh.center
+        >>> trans = mesh.translate((2, 1, 2), inplace=False)
+        >>> trans.center
         [2.0, 1.0, 2.0]
 
         """
-        self.points += np.asarray(xyz)  # type: ignore
+        transform = _vtk.vtkTransform()
+        transform.Translate(xyz)
+        return self.transform(transform, transform_all_input_vectors=transform_all_input_vectors, inplace=inplace)
 
-    def scale(self, xyz: Union[list, tuple, np.ndarray]):
+    def scale(self, xyz: Union[list, tuple, np.ndarray], transform_all_input_vectors=False, inplace=False):
         """Scale the mesh.
 
+        .. note::
+            See also the notes at :func:`transform()
+            <DataSetFilters.transform>` which is used by this filter
+            under the hood.
+
         Parameters
         ----------
         xyz : scale factor list or tuple or np.ndarray
             Length 3 list, tuple or array.
 
+        transform_all_input_vectors : bool, optional
+            When ``True``, all input vectors are
+            transformed. Otherwise, only the points, normals and
+            active vectors are transformed.
+
+        inplace : bool, optional
+            Updates mesh in-place.
+
+        Returns
+        -------
+        pyvista.DataSet
+            Scaled dataset.
+
         Examples
         --------
         >>> import pyvista
@@ -1012,16 +1172,22 @@ class DataSet(DataSetFilters, DataObject):
         >>> pl.subplot(0, 1)
         >>> pl.show_axes()
         >>> _ = pl.show_grid()
-        >>> mesh2 = mesh1.copy()
-        >>> mesh2.scale([10.0, 10.0, 10.0])
+        >>> mesh2 = mesh1.scale([10.0, 10.0, 10.0], inplace=False)
         >>> _ = pl.add_mesh(mesh2)
         >>> pl.show(cpos="xy")
         """
-        self.points *= np.asarray(xyz)  # type: ignore
+        transform = _vtk.vtkTransform()
+        transform.Scale(xyz)
+        return self.transform(transform, transform_all_input_vectors=transform_all_input_vectors, inplace=inplace)
 
-    def flip_x(self, point=None, transform_all_input_vectors=False):
+    def flip_x(self, point=None, transform_all_input_vectors=False, inplace=False):
         """Flip mesh about the x-axis.
 
+        .. note::
+            See also the notes at :func:`transform()
+            <DataSetFilters.transform>` which is used by this filter
+            under the hood.
+
         Parameters
         ----------
         point : list, optional
@@ -1033,6 +1199,14 @@ class DataSet(DataSetFilters, DataObject):
             transformed. Otherwise, only the points, normals and
             active vectors are transformed.
 
+        inplace : bool, optional
+            Updates mesh in-place.
+
+        Returns
+        -------
+        pyvista.DataSet
+            Flipped dataset.
+
         Examples
         --------
         >>> import pyvista
@@ -1044,8 +1218,7 @@ class DataSet(DataSetFilters, DataObject):
         >>> _ = pl.add_mesh(mesh1)
         >>> pl.subplot(0, 1)
         >>> pl.show_axes()
-        >>> mesh2 = mesh1.copy()
-        >>> mesh2.flip_x()
+        >>> mesh2 = mesh1.flip_x(inplace=False)
         >>> _ = pl.add_mesh(mesh2)
         >>> pl.show(cpos="xy")
         """
@@ -1053,11 +1226,16 @@ class DataSet(DataSetFilters, DataObject):
             point = self.center
         check_valid_vector(point, 'point')
         t = transformations.reflection((1, 0, 0), point=point)
-        self.transform(t, transform_all_input_vectors=transform_all_input_vectors, inplace=True)
+        return self.transform(t, transform_all_input_vectors=transform_all_input_vectors, inplace=inplace)
 
-    def flip_y(self, point=None, transform_all_input_vectors=False):
+    def flip_y(self, point=None, transform_all_input_vectors=False, inplace=False):
         """Flip mesh about the y-axis.
 
+        .. note::
+            See also the notes at :func:`transform()
+            <DataSetFilters.transform>` which is used by this filter
+            under the hood.
+
         Parameters
         ----------
         point : list, optional
@@ -1069,6 +1247,14 @@ class DataSet(DataSetFilters, DataObject):
             transformed. Otherwise, only the points, normals and
             active vectors are transformed.
 
+        inplace : bool, optional
+            Updates mesh in-place.
+
+        Returns
+        -------
+        pyvista.DataSet
+            Flipped dataset.
+
         Examples
         --------
         >>> import pyvista
@@ -1080,8 +1266,7 @@ class DataSet(DataSetFilters, DataObject):
         >>> _ = pl.add_mesh(mesh1)
         >>> pl.subplot(0, 1)
         >>> pl.show_axes()
-        >>> mesh2 = mesh1.copy()
-        >>> mesh2.flip_y()
+        >>> mesh2 = mesh1.flip_y(inplace=False)
         >>> _ = pl.add_mesh(mesh2)
         >>> pl.show(cpos="xy")
         """
@@ -1089,11 +1274,16 @@ class DataSet(DataSetFilters, DataObject):
             point = self.center
         check_valid_vector(point, 'point')
         t = transformations.reflection((0, 1, 0), point=point)
-        self.transform(t, transform_all_input_vectors=transform_all_input_vectors, inplace=True)
+        return self.transform(t, transform_all_input_vectors=transform_all_input_vectors, inplace=inplace)
 
-    def flip_z(self, point=None, transform_all_input_vectors=False):
+    def flip_z(self, point=None, transform_all_input_vectors=False, inplace=False):
         """Flip mesh about the z-axis.
 
+        .. note::
+            See also the notes at :func:`transform()
+            <DataSetFilters.transform>` which is used by this filter
+            under the hood.
+
         Parameters
         ----------
         point : list, optional
@@ -1105,6 +1295,14 @@ class DataSet(DataSetFilters, DataObject):
             transformed. Otherwise, only the points, normals and
             active vectors are transformed.
 
+        inplace : bool, optional
+            Updates mesh in-place.
+
+        Returns
+        -------
+        pyvista.DataSet
+            Flipped dataset.
+
         Examples
         --------
         >>> import pyvista
@@ -1116,8 +1314,7 @@ class DataSet(DataSetFilters, DataObject):
         >>> _ = pl.add_mesh(mesh1)
         >>> pl.subplot(0, 1)
         >>> pl.show_axes()
-        >>> mesh2 = mesh1.copy()
-        >>> mesh2.flip_z()
+        >>> mesh2 = mesh1.flip_z(inplace=False)
         >>> _ = pl.add_mesh(mesh2)
         >>> pl.show(cpos="xy")
         """
@@ -1125,11 +1322,16 @@ class DataSet(DataSetFilters, DataObject):
             point = self.center
         check_valid_vector(point, 'point')
         t = transformations.reflection((0, 0, 1), point=point)
-        self.transform(t, transform_all_input_vectors=transform_all_input_vectors, inplace=True)
+        return self.transform(t, transform_all_input_vectors=transform_all_input_vectors, inplace=inplace)
 
-    def flip_normal(self, normal: List[float], point=None, transform_all_input_vectors=False):
+    def flip_normal(self, normal: List[float], point=None, transform_all_input_vectors=False, inplace=False):
         """Flip mesh about the normal.
 
+        .. note::
+            See also the notes at :func:`transform()
+            <DataSetFilters.transform>` which is used by this filter
+            under the hood.
+
         Parameters
         ----------
         normal : tuple
@@ -1144,6 +1346,14 @@ class DataSet(DataSetFilters, DataObject):
             transformed. Otherwise, only the points, normals and
             active vectors are transformed.
 
+        inplace : bool, optional
+            Updates mesh in-place.
+
+        Returns
+        -------
+        pyvista.DataSet
+            Dataset flipped about its normal.
+
         Examples
         --------
         >>> import pyvista
@@ -1155,8 +1365,7 @@ class DataSet(DataSetFilters, DataObject):
         >>> _ = pl.add_mesh(mesh1)
         >>> pl.subplot(0, 1)
         >>> pl.show_axes()
-        >>> mesh2 = mesh1.copy()
-        >>> mesh2.flip_normal([1.0, 1.0, 1.0])
+        >>> mesh2 = mesh1.flip_normal([1.0, 1.0, 1.0], inplace=False)
         >>> _ = pl.add_mesh(mesh2)
         >>> pl.show(cpos="xy")
         """
@@ -1165,7 +1374,7 @@ class DataSet(DataSetFilters, DataObject):
         check_valid_vector(normal, 'normal')
         check_valid_vector(point, 'point')
         t = transformations.reflection(normal, point=point)
-        self.transform(t, transform_all_input_vectors=transform_all_input_vectors, inplace=True)
+        return self.transform(t, transform_all_input_vectors=transform_all_input_vectors, inplace=inplace)
 
     def copy_meta_from(self, ido: 'DataSet'):
         """Copy pyvista meta data onto this object from another object.
@@ -1206,7 +1415,7 @@ class DataSet(DataSetFilters, DataObject):
 
         >>> import pyvista
         >>> import numpy as np
-        >>> mesh = pyvista.Cube().clean()
+        >>> mesh = pyvista.Cube()
         >>> mesh.clear_data()
         >>> mesh.point_data['my_array'] = np.random.random(mesh.n_points)
         >>> mesh.point_data['my_other_array'] = np.arange(mesh.n_points)
@@ -1342,11 +1551,11 @@ class DataSet(DataSetFilters, DataObject):
 
         Examples
         --------
-        Add cell arrays to a mesh and list the available ``cell_data``. 
+        Add cell arrays to a mesh and list the available ``cell_data``.
 
         >>> import pyvista
         >>> import numpy as np
-        >>> mesh = pyvista.Cube().clean()
+        >>> mesh = pyvista.Cube()
         >>> mesh.clear_data()
         >>> mesh.cell_data['my_array'] = np.random.random(mesh.n_cells)
         >>> mesh.cell_data['my_other_array'] = np.arange(mesh.n_cells)
@@ -1385,7 +1594,7 @@ class DataSet(DataSetFilters, DataObject):
         mesh.
 
         >>> import pyvista
-        >>> cube = pyvista.Cube().clean()
+        >>> cube = pyvista.Cube()
         >>> cube.n_points
         8
 
@@ -1407,7 +1616,7 @@ class DataSet(DataSetFilters, DataObject):
         mesh.
 
         >>> import pyvista
-        >>> cube = pyvista.Cube().clean()
+        >>> cube = pyvista.Cube()
         >>> cube.n_cells
         6
 
@@ -1425,22 +1634,22 @@ class DataSet(DataSetFilters, DataObject):
         return self.GetNumberOfCells()
 
     @property
-    def bounds(self) -> List[float]:
+    def bounds(self) -> Tuple[float, float, float, float, float, float]:
         """Return the bounding box of this dataset.
 
-        The form is: ``[xmin, xmax, ymin, ymax, zmin, zmax]``.
+        The form is: ``(xmin, xmax, ymin, ymax, zmin, zmax)``.
 
         Examples
         --------
         Create a cube and return the bounds of the mesh.
 
         >>> import pyvista
-        >>> cube = pyvista.Cube().clean()
+        >>> cube = pyvista.Cube()
         >>> cube.bounds
-        [-0.5, 0.5, -0.5, 0.5, -0.5, 0.5]
+        (-0.5, 0.5, -0.5, 0.5, -0.5, 0.5)
 
         """
-        return list(self.GetBounds())
+        return self.GetBounds()
 
     @property
     def length(self) -> float:
@@ -1477,16 +1686,16 @@ class DataSet(DataSetFilters, DataObject):
         return list(self.GetCenter())
 
     @property
-    def extent(self) -> Optional[list]:
+    def extent(self) -> Optional[tuple]:
         """Return the range of the bounding box."""
         try:
-            _extent = list(self.GetExtent())
+            _extent = self.GetExtent()
         except AttributeError:
             return None
         return _extent
 
     @extent.setter
-    def extent(self, extent: List[float]):
+    def extent(self, extent: Sequence[float]):
         """Set the range of the bounding box."""
         if hasattr(self, 'SetExtent'):
             if len(extent) != 6:
@@ -1540,7 +1749,7 @@ class DataSet(DataSetFilters, DataObject):
         Create a DataSet with a variety of arrays.
 
         >>> import pyvista
-        >>> mesh = pyvista.Cube().clean()
+        >>> mesh = pyvista.Cube()
         >>> mesh.clear_data()
         >>> mesh.point_data['point-data'] = range(mesh.n_points)
         >>> mesh.cell_data['cell-data'] = range(mesh.n_cells)
@@ -1593,7 +1802,7 @@ class DataSet(DataSetFilters, DataObject):
         Create a DataSet with a variety of arrays.
 
         >>> import pyvista
-        >>> mesh = pyvista.Cube().clean()
+        >>> mesh = pyvista.Cube()
         >>> mesh.clear_data()
         >>> mesh.point_data['point-data'] = range(mesh.n_points)
         >>> mesh.cell_data['cell-data'] = range(mesh.n_cells)
@@ -1779,7 +1988,7 @@ class DataSet(DataSetFilters, DataObject):
 
         >>> import pyvista
         >>> mesh_a = pyvista.Sphere()
-        >>> mesh_b = pyvista.Cube().clean()
+        >>> mesh_b = pyvista.Cube()
         >>> mesh_a.overwrite(mesh_b)
         >>> mesh_a == mesh_b
         True
@@ -1880,8 +2089,8 @@ class DataSet(DataSetFilters, DataObject):
         Parameters
         ----------
         point : iterable(float) or np.ndarray
-            Length 3 coordinate of the point to query or a ``numpy`` array
-            of coordinates.
+            Coordinates of point to query (length 3) or a ``numpy`` array of ``n``
+            points with shape ``(n, 3)``.
 
         Returns
         -------
@@ -1889,25 +2098,41 @@ class DataSet(DataSetFilters, DataObject):
             Index or indices of the cell in this mesh that is closest
             to the given point.
 
+        Warnings
+        --------
+        This method may still return a valid cell index even if the point
+        contains a value like ``numpy.inf`` or ``numpy.nan``.
+
         Examples
         --------
-        Find nearest cell to a point on a sphere
+        Find nearest cell on a sphere centered on the
+        origin to the point ``[0.1, 0.2, 0.3]``.
 
         >>> import pyvista
         >>> mesh = pyvista.Sphere()
-        >>> index = mesh.find_closest_cell([0, 0, 0.5])
+        >>> point = [0.1, 0.2, 0.3]
+        >>> index = mesh.find_closest_cell(point)
         >>> index
-        59
+        591
 
-        Find the nearest cells to several random points.  Note that
-        ``-1`` indicates that the locator was not able to find a
-        reasonably close cell.
+        Make sure that this cell indeed is the closest to
+        ``[0.1, 0.2, 0.3]``.
 
         >>> import numpy as np
-        >>> points = np.random.random((1000, 3))
+        >>> cell_centers = mesh.cell_centers()
+        >>> relative_position = cell_centers.points - point
+        >>> distance = np.linalg.norm(relative_position, axis=1)
+        >>> np.argmin(distance)
+        591
+
+        Find the nearest cells to several random points that
+        are centered on the origin.
+
+        >>> points = 2 * np.random.random((5000, 3)) - 1
         >>> indices = mesh.find_closest_cell(points)
         >>> indices.shape
-        (1000,)
+        (5000,)
+
         """
         if isinstance(point, collections.abc.Sequence):
             point = np.array(point)
@@ -1928,8 +2153,93 @@ class DataSet(DataSetFilters, DataObject):
         locator = _vtk.vtkCellLocator()
         locator.SetDataSet(self)
         locator.BuildLocator()
-        closest_cells = np.array([locator.FindCell(node) for node in point])
-        return int(closest_cells[0]) if len(closest_cells) == 1 else closest_cells
+
+        cell = _vtk.vtkGenericCell()
+        closest_point = [0, 0, 0]
+        cellId = _vtk.mutable(0)
+        subld = _vtk.mutable(0)
+        dist2 = _vtk.mutable(0.0)
+
+        closest_cells = []
+        for node in point:
+            locator.FindClosestPoint(node, closest_point, cell, cellId, subld, dist2)
+            closest_cells.append(int(cellId))
+        return closest_cells[0] if len(closest_cells) == 1 else np.array(closest_cells)
+
+    def find_cells_along_line(
+        self,
+        pointa: Iterable[float],
+        pointb: Iterable[float],
+        tolerance=0.0,
+    ) -> np.ndarray:
+        """Find the index of cells in this mesh along a line.
+
+        Line is defined from ``pointa`` to ``pointb``.
+
+        Parameters
+        ----------
+        pointa : iterable(float)
+            Length 3 coordinate of the start of the line.
+
+        pointb : iterable(float)
+            Length 3 coordinate of the end of the line.
+
+        tolerance : float, optional
+            The absolute tolerance to use to find cells along line.
+
+        Returns
+        -------
+        numpy.ndarray
+            Index or indices of the cell in this mesh that are closest
+            to the given point.
+
+        Examples
+        --------
+        >>> import pyvista
+        >>> mesh = pyvista.Sphere()
+        >>> index = mesh.find_cells_along_line([0, 0, 0], [0, 0, 1.0])
+
+        """
+        if np.array(pointa).size != 3:
+            raise TypeError("Point A must be a length three tuple of floats.")
+        if np.array(pointb).size != 3:
+            raise TypeError("Point B must be a length three tuple of floats.")
+        locator = _vtk.vtkCellLocator()
+        locator.SetDataSet(self)
+        locator.BuildLocator()
+        id_list = _vtk.vtkIdList()
+        locator.FindCellsAlongLine(pointa, pointb, tolerance, id_list)
+        return vtk_id_list_to_array(id_list)
+
+    def find_cells_within_bounds(self, bounds: Iterable[float]) -> np.ndarray:
+        """Find the index of cells in this mesh within bounds.
+
+        Parameters
+        ----------
+        bounds : iterable(float)
+            Bounding box. The form is: ``[xmin, xmax, ymin, ymax, zmin, zmax]``.
+
+        Returns
+        -------
+        numpy.ndarray
+            Index or indices of the cell in this mesh that are closest
+            to the given point.
+
+        Examples
+        --------
+        >>> import pyvista
+        >>> mesh = pyvista.Cube()
+        >>> index = mesh.find_cells_within_bounds([-2.0, 2.0, -2.0, 2.0, -2.0, 2.0])
+
+        """
+        if np.array(bounds).size != 6:
+            raise TypeError("Bounds must be a length three tuple of floats.")
+        locator = _vtk.vtkCellTreeLocator()
+        locator.SetDataSet(self)
+        locator.BuildLocator()
+        id_list = _vtk.vtkIdList()
+        locator.FindCellsWithinBounds(list(bounds), id_list)
+        return vtk_id_list_to_array(id_list)
 
     def cell_n_points(self, ind: int) -> int:
         """Return the number of points in a cell.
@@ -1984,7 +2294,7 @@ class DataSet(DataSetFilters, DataObject):
         points = _vtk.vtk_to_numpy(points)
         return points.copy()
 
-    def cell_bounds(self, ind: int) -> List[float]:
+    def cell_bounds(self, ind: int) -> Tuple[float, float, float, float, float, float]:
         """Return the bounding box of a cell.
 
         Parameters
@@ -1994,7 +2304,7 @@ class DataSet(DataSetFilters, DataObject):
 
         Returns
         -------
-        list(float)
+        tuple(float)
             The limits of the cell in the X, Y and Z directions respectivelly.
 
         Examples
@@ -2002,10 +2312,10 @@ class DataSet(DataSetFilters, DataObject):
         >>> from pyvista import examples
         >>> mesh = examples.load_airplane()
         >>> mesh.cell_bounds(0)
-        [896.9940185546875, 907.5390014648438, 48.760101318359375, 55.49020004272461, 80.74520111083984, 83.65809631347656]
+        (896.9940185546875, 907.5390014648438, 48.760101318359375, 55.49020004272461, 80.74520111083984, 83.65809631347656)
 
         """
-        return list(self.GetCell(ind).GetBounds())
+        return self.GetCell(ind).GetBounds()
 
     def cell_type(self, ind: int) -> int:
         """Return the type of a cell.
