@@ -1,11 +1,19 @@
 """Module dedicated to widgets."""
 
+from typing import List
+
 import numpy as np
 
 import pyvista
 from pyvista import _vtk
-from pyvista.utilities import (NORMALS, generate_plane, get_array,
-                               try_callback, get_array_association)
+from pyvista.utilities import (
+    NORMALS,
+    generate_plane,
+    get_array,
+    get_array_association,
+    try_callback,
+)
+
 from .tools import parse_color
 
 
@@ -16,6 +24,8 @@ class WidgetHelper:
 
     """
 
+    _camera_widgets: List[object] = []
+
     def add_box_widget(self, callback, bounds=None, factor=1.25,
                        rotation_enabled=True, color=None, use_planes=False,
                        outline_translation=True, pass_widget=False):
@@ -613,7 +623,7 @@ class WidgetHelper:
         ----------
         mesh : pyvista.DataSet
             The input dataset to add to the scene and threshold.
-        
+
         generate_triangles : bool, optional
             If this is enabled (``False`` by default), the output will be
             triangles otherwise, the output will be the intersection polygons.
@@ -1688,7 +1698,7 @@ class WidgetHelper:
             color3 = np.array(parse_color(color3)) * 255
 
             n_points = dims[0] * dims[1]
-            button = pyvista.UniformGrid(dims)
+            button = pyvista.UniformGrid(dims=dims)
             arr = np.array([color1] * n_points).reshape(dims[0], dims[1], 3)  # fill with color1
             arr[1:dims[0]-1, 1:dims[1]-1] = color2  # apply color2
             arr[
@@ -1730,6 +1740,50 @@ class WidgetHelper:
         self.button_widgets.append(button_widget)
         return button_widget
 
+    def add_camera_orientation_widget(self, animate=True, n_frames=20):
+        """Add a camera orientation widget to the active renderer.
+
+        .. note::
+           This widget requires ``vtk>=9.1.0``.
+
+        Parameters
+        ----------
+        animate : bool, optional
+            Enable or disable jump-to-axis-view animation.
+        n_frames : int, optional
+            The number of frames to animate the jump-to-axis-viewpoint feature.
+
+        Returns
+        -------
+        vtkCameraOrientationWidget
+            Camera orientation widget.
+
+        Examples
+        --------
+        Add a camera orientation widget to the scene.
+
+        >>> import pyvista
+        >>> mesh = pyvista.Cube()
+        >>> plotter = pyvista.Plotter()
+        >>> _ = plotter.add_mesh(mesh, scalars=range(6), show_scalar_bar=False)
+        >>> _ = plotter.add_camera_orientation_widget()
+        >>> plotter.show()
+
+        """
+        widget = _vtk.lazy_vtkCameraOrientationWidget()
+        widget.SetParentRenderer(self.renderer)
+        widget.SetAnimate(animate)
+        widget.SetAnimatorTotalFrames(n_frames)
+        widget.On()
+        self._camera_widgets.append(widget)
+        return widget
+
+    def clear_camera_widgets(self):
+        """Disable all of the camera widgets."""
+        for widget in self._camera_widgets:
+            widget.Off()
+        self._camera_widgets = []
+
     def clear_button_widgets(self):
         """Disable all of the button widgets."""
         if hasattr(self, 'button_widgets'):
@@ -1746,3 +1800,4 @@ class WidgetHelper:
         self.clear_sphere_widgets()
         self.clear_spline_widgets()
         self.clear_button_widgets()
+        self.clear_camera_widgets()
